@@ -1,2 +1,3 @@
 # a temporary GUI string reporting file
-GUIString = "foo"
\ No newline at end of file
+GUIString = ""
+GUISubtypeString = ""
\ No newline at end of file
