@@ -26,7 +26,7 @@ from core.constants import DEVICE_TYPE_SMARTPHONE
 #N900 specific:
 import dbus.glib
 
-from core import gs
+from core import gs, constants
 # only import GTK, Hildon & Liblocation when using GTK GUI
 if gs.GUIString == "GTK":
     import hildon
@@ -73,7 +73,9 @@ class DeviceN900(DeviceModule):
         print("N900: DBUS initialized")
 
         # Internet connectivity related
-        self.connectivityStatus = False
+
+        # status from the Internet Connectivity Daemon
+        self._connectivityStatusICD = constants.OFFLINE
         self.conicConnection = None
 
         # Mainloop for headless location support
@@ -339,7 +341,7 @@ class DeviceN900(DeviceModule):
         optionsButton = gtk.Button("Options")
         optionsButton.connect('clicked', self._switchToMenu, 'options')
         searchButton = gtk.Button("Search")
-        searchButton.connect('clicked', self._switchToMenu, 'search')
+        searchButton.connect('clicked', self._switchToMenu, 'searchWhat')
         routeButton = gtk.Button("Route")
         routeButton.connect('clicked', self._switchToMenu, 'route')
 
@@ -604,28 +606,42 @@ class DeviceN900(DeviceModule):
 
     # ** Internet connectivity **
 
-    def getInternetConnectivityStatus(self):
-        """report Internet connectivity status based on data from libconic"""
-        return self.connectivityStatus
-
     def _connectionStateCB(self, connection, event):
         """handle Internet connectivity state changes"""
-        #    print("connection_cb(%s, %s)" % (connection, event))
-        status = event.get_status()
-        error = event.get_error()
-        iap_id = event.get_iap_id()
-        bearer = event.get_bearer_type()
-
-        if status == conic.STATUS_CONNECTED:
-            self.connectivityStatus = True
-        elif status == conic.STATUS_DISCONNECTED:
-            self.connectivityStatus = False
-        elif status == conic.STATUS_DISCONNECTING:
-            self.connectivityStatus = False
+        #print("connection_cb(%s, %s)" % (connection, event))
+        conic_status = event.get_status()
+        #print(conic_status)
+        #error = event.get_error()
+        #iap_id = event.get_iap_id()
+        #bearer = event.get_bearer_type()
+        status = constants.CONNECTIVITY_UNKNOWN
+        if conic_status == conic.STATUS_CONNECTED:
+            status = constants.ONLINE
+            #print("CONIC CONNECTED")
+        elif conic_status == conic.STATUS_DISCONNECTED:
+            status = constants.OFFLINE
+            #print("CONIC DISCONNECTED")
+        elif conic_status == conic.STATUS_DISCONNECTING:
+            status = constants.OFFLINE
+            #print("CONIC DISCONNECTING")
+        self._connectivityStatusICD = status
+        # trigger the connectivity status changed signal
+        self.internetConnectivityChanged(status)
 
     def enableInternetConnectivity(self):
-        """autoconnect to the Internet using DBUS"""
-        subprocess.call(["sh", "modules/device_modules/n900_maemo5_autoconnect_dbus.sh"])
+        """Autoconnect to the Internet using DBUS"""
+        # if connectivity is requested like this,
+        # the callback will be called at once if we are online,
+        # so current connectivity state can be determined
+        self.conicConnection.request_connection(conic.CONNECT_FLAG_AUTOMATICALLY_TRIGGERED)
+
+    @property
+    def connectivityStatus(self):
+        # The conic/ICD callback is called both once registered
+        # and also when trying to enable connectivity, so we can use data
+        # from the callback to represent the connectivity state,
+        # overriding the portable connectivityStatus implementation in base_device_module
+        return self._connectivityStatusICD
 
     def getDeviceType(self):
         return DEVICE_TYPE_SMARTPHONE
