@@ -6,8 +6,8 @@
 api_major_version = 5   # bumped on incompatible changes
 api_minor_version = 0   # bumped on compatible changes
 
-from gps import *
-from misc import *
+from .gps import *
+from .misc import *
 
 # The 'client' module exposes some C utility functions for Python clients.
 # The 'packet' module exposes the packet getter via a Python interface.
\ No newline at end of file
